
/* Chrysalide - Outil d'analyse de fichiers binaires
 * registers.c - aides auxiliaires relatives aux registres Dalvik
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "register.h"


#include <stdio.h>


#include "../register-int.h"
#include "../sharing/manager.h"
#include "../../common/sort.h"



/* ------------------------- ENCADREMENT DE REGISTRES BRUTS ------------------------- */


/* Représentation d'un registre Dalvik (instance) */
struct _GDalvikRegister
{
    GArchRegister parent;                   /* Instance parente            */

    uint16_t index;                         /* Indice du registre          */

};


/* Représentation d'un registre Dalvik (classe) */
struct _GDalvikRegisterClass
{
    GArchRegisterClass parent;              /* Classe parente              */

};


#define MAX_REGNAME_LEN 8


/* Initialise la classe des registres Dalvik. */
static void g_dalvik_register_class_init(GDalvikRegisterClass *);

/* Initialise une instance de registre Dalvik. */
static void g_dalvik_register_init(GDalvikRegister *);

/* Supprime toutes les références externes. */
static void g_dalvik_register_dispose(GDalvikRegister *);

/* Procède à la libération totale de la mémoire. */
static void g_dalvik_register_finalize(GDalvikRegister *);

/* Initialise un nouvel objet partagé avec des informations. */
static bool g_dalvik_register_do_init(GDalvikRegister *, const GDalvikRegister *);

/* Produit une empreinte à partir d'un registre. */
static guint g_dalvik_register_hash(const GDalvikRegister *);

/* Traduit un registre en version humainement lisible. */
static void g_dalvik_register_print(const GDalvikRegister *, GBufferLine *, AsmSyntax);



/* -------------------------- PARTAGES DE CONTENUS UNIQUES -------------------------- */


/* Gestionnaire des partages d'instances */
static GShareManager *_dalvik_register_manager = NULL;



/* ---------------------------------------------------------------------------------- */
/*                           ENCADREMENT DE REGISTRES BRUTS                           */
/* ---------------------------------------------------------------------------------- */


/* Indique le type défini pour une représentation d'un registre Dalvik. */
G_DEFINE_TYPE(GDalvikRegister, g_dalvik_register, G_TYPE_ARCH_REGISTER);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des registres Dalvik.                   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_register_class_init(GDalvikRegisterClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GArchRegisterClass *register_class;     /* Classe de haut niveau       */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_dalvik_register_dispose;
    object->finalize = (GObjectFinalizeFunc)g_dalvik_register_finalize;

    register_class = G_ARCH_REGISTER_CLASS(klass);

    register_class->init = (init_shared_fc)g_dalvik_register_do_init;

    register_class->hash = (reg_hash_fc)g_dalvik_register_hash;
    register_class->compare = (reg_compare_fc)g_dalvik_register_compare;
    register_class->print = (reg_print_fc)g_dalvik_register_print;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reg = instance à initialiser.                                *
*                                                                             *
*  Description : Initialise une instance de registre Dalvik.                  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_register_init(GDalvikRegister *reg)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reg = instance d'objet GLib à traiter.                       *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_register_dispose(GDalvikRegister *reg)
{
    G_OBJECT_CLASS(g_dalvik_register_parent_class)->dispose(G_OBJECT(reg));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reg = instance d'objet GLib à traiter.                       *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_register_finalize(GDalvikRegister *reg)
{
    G_OBJECT_CLASS(g_dalvik_register_parent_class)->finalize(G_OBJECT(reg));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : index = indice du registre correspondant.                    *
*                                                                             *
*  Description : Crée une réprésentation de registre Dalvik.                  *
*                                                                             *
*  Retour      : Adresse de la structure mise en place.                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDalvikRegister *g_dalvik_register_new(uint16_t index)
{
    GDalvikRegister *result;                /* Structure à retourner       */
    GDalvikRegister fake;                   /* Transport d'informations    */

    fake.index = index;

    result = G_DALVIK_REGISTER(g_share_manager_get(_dalvik_register_manager, (GSharedInstance *)&fake));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reg  = objet partagé à initialiser.                          *
*                fake = coquille vide contenant les infos à enregistrer.      *
*                                                                             *
*  Description : Initialise un nouvel objet partagé avec des informations.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool g_dalvik_register_do_init(GDalvikRegister *reg, const GDalvikRegister *fake)
{
    reg->index = fake->index;

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reg = opérande à consulter pour le calcul.                   *
*                                                                             *
*  Description : Produit une empreinte à partir d'un registre.                *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static guint g_dalvik_register_hash(const GDalvikRegister *reg)
{
    return reg->index;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reg    = registre à transcrire.                              *
*                line   = ligne tampon où imprimer l'opérande donné.          *
*                syntax = type de représentation demandée.                    *
*                                                                             *
*  Description : Traduit un registre en version humainement lisible.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dalvik_register_print(const GDalvikRegister *reg, GBufferLine *line, AsmSyntax syntax)
{
    char key[MAX_REGNAME_LEN];              /* Mot clef principal          */
    size_t klen;                            /* Taille de ce mot clef       */

    switch (syntax)
    {
        case ASX_INTEL:
            klen = snprintf(key, MAX_REGNAME_LEN, "v%hu", reg->index);
            break;

        case ASX_ATT:
            klen = snprintf(key, MAX_REGNAME_LEN, "%%v%hu", reg->index);
            break;

        default:
            klen = 0;
            break;

    }

    g_buffer_line_append_text(line, BLC_ASSEMBLY, key, klen, RTT_REGISTER, NULL);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : reg = registre à consulter.                                  *
*                                                                             *
*  Description : Fournit l'indice d'un registre Dalvik.                       *
*                                                                             *
*  Retour      : Inditifiant représentant le registre.                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

uint16_t g_dalvik_register_get_index(const GDalvikRegister *reg)
{
    return reg->index;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : a = premier opérande à consulter.                            *
*                b = second opérande à consulter.                             *
*                                                                             *
*  Description : Compare un registre avec un autre.                           *
*                                                                             *
*  Retour      : Bilan de la comparaison.                                     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

int g_dalvik_register_compare(const GDalvikRegister * const *a, const GDalvikRegister * const *b)
{
    int result;                             /* Bilan à retourner           */

    result = sort_unsigned_long((*a)->index, (*b)->index);

    return result;

}


/* ---------------------------------------------------------------------------------- */
/*                            PARTAGES DE CONTENUS UNIQUES                            */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Met en place les mécanismes de partage des registres Dalvik. *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool init_dalvik_register_sharing(void)
{
    _dalvik_register_manager = g_share_manager_new(G_TYPE_DALVIK_REGISTER);

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Imprime des statistiques quant aux partages dans l'archi.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/
#ifdef DEBUG_DUMP_STATS
void dump_dalvik_register_share_stats(void)
{
    g_share_manager_dump_stats(_dalvik_register_manager);

}
#endif


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Supprime les mécanismes de partage des registres Dalvik.     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void exit_dalvik_register_sharing(void)
{
    g_object_unref(G_OBJECT(_dalvik_register_manager));

}
